@@ -1,6 +1,6 @@
 from collections import OrderedDict
 
-from common import AbstractWritableDataStore
+from .common import AbstractWritableDataStore
 
 
 class InMemoryDataStore(AbstractWritableDataStore):
@@ -26,9 +26,3 @@ class InMemoryDataStore(AbstractWritableDataStore):
 
     def del_attribute(self, key):
         del self.attributes[key]
-
-    def sync(self):
-        pass
-
-    def close(self):
-        pass
